import os
from unittest import TestCase, mock

from jovian._version import __version__
from jovian.tests.resources import fake_creds
from jovian.utils import slack
from jovian.utils.credentials import write_creds
from jovian.utils.error import ApiError
from jovian.utils.slack import _h, add_slack, notify


def test_h():
    with fake_creds('.jovian', 'credentials.json'):
        expected_result = {"Authorization": "Bearer fake_api_key",
                           "x-jovian-source": "library",
                           "x-jovian-library-version": __version__,
                           "x-jovian-command": "add-slack",
                           "x-jovian-guest": "b6538d4dfde04fcf993463a828a9cec6",
                           "x-jovian-org": "staging"}

        assert _h() == expected_result


class MockResponse:
    def __init__(self, json_data, status_code, text=""):
        self.json_data = json_data
        self.status_code = status_code
        self.text = text

    def json(self):
        return self.json_data


def mock_requests_get(url, *args, **kwargs):
    if url == 'https://api-staging.jovian.ai/slack/integration_details':
        if kwargs['headers']['Authorization'] == 'Bearer fake_api_key':
            data = {"data": {
                "slackAccount": {
                    "accountId": 47,
                    "channel": "@rohit",
                    "deleted": False,
                    "id": 10,
                    "updatedAt": "Fri, 13 Dec 2019 13:41:27 GMT",
                    "workspace": "jovian.ml"
                }
            },
                "success": True}

            return MockResponse(data, status_code=200)

        elif kwargs['headers']['Authorization'] == 'Bearer fake_api_key_error':
            data = {"errors": [
                {
                    "code": 200,
                    "message": "Invalid guest key"
                }
            ],
                "success": False}
            return MockResponse(data, 200)

        else:
            data = {"errors": [
                {
                    "code": 401,
                    "message": "Missing Authorization Header"
                }
            ],
                "success": False}
            return MockResponse(data, 401)


def mock_requests_post(url, *args, **kwargs):
    if url == 'https://api-staging.jovian.ai/slack/notify':
        if kwargs['headers']['Authorization'] == 'Bearer fake_api_key':
            data = {
                "data": {
                    "messageId": "60023f08f3b54801bd58cf6b37067ed6",
                    "messageSent": True
                },
                "success": True
            }
            return MockResponse(data, 200)
        elif kwargs['headers']['Authorization'] == 'Bearer fake_invalid_api_key':
            data = {
                "errors": [
                    {
                        "code": 401,
                        "message": "The token is invalid"
                    }
                ],
                "success": False
            }

            return MockResponse(data, 401)

        elif kwargs['headers']['Authorization'] == 'Bearer fake_expired_api_key':
            data = {
                "errors": [
                    {
                        "code": 200,
                        "message": "The token has expired"
                    }
                ],
                "success": False
            }

            return MockResponse(data, 200)


@mock.patch("jovian.utils.slack.get", side_effect=mock_requests_get)
def test_add_slack(mock_get, capsys):
    with fake_creds('.jovian-add-slack', 'credentials.json'):
        # setUp
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "API_URL": "https://api-staging.jovian.ai",
                 "API_KEY": "fake_api_key",
                 "ORG_ID": "staging"}
        write_creds(creds)

        add_slack()

        captured = capsys.readouterr()

        assert captured.out.strip() == "[jovian] Slack already connected." + \
                                       " \nWorkspace: jovian.ml\nConnected Channel: @rohit"


@mock.patch("jovian.utils.slack.get", side_effect=mock_requests_get)
def test_add_slack_errors(mock_get, capsys):
    with fake_creds('.jovian-add-slack', 'credentials.json'):
        # setUp
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "API_URL": "https://api-staging.jovian.ai",
                 "API_KEY": "fake_api_key_error",
                 "ORG_ID": "staging"}
        write_creds(creds)

        add_slack()

        captured = capsys.readouterr()

        assert captured.out.strip() == "[jovian] Invalid guest key"


class TestAddSlack(TestCase):
    @mock.patch("jovian.utils.slack.get", side_effect=mock_requests_get)
    def test_add_slack_api_error(self, mock_get):
        with fake_creds('.jovian-add-slack', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            with self.assertRaises(ApiError):
                add_slack()


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_notify(mock_requests_post, capsys):
    with fake_creds('.jovian-notify', 'credentials.json'):
        # setUp
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "API_URL": "https://api-staging.jovian.ai",
                 "API_KEY": "fake_api_key",
                 "ORG_ID": "staging"}
        write_creds(creds)

        data = {'key': 'value'}
        notify(data)

        captured = capsys.readouterr()

        assert captured.out.strip() == "[jovian] message_sent:True"


@mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
@mock.patch("requests.post", side_effect=mock_requests_post)
def test_notify_safe_true(mock_requests_post, mock_get_api_key, capsys):
    with fake_creds('.jovian-notify', 'credentials.json'):
        # setUp
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "API_URL": "https://api-staging.jovian.ai",
                 "API_KEY": "fake_invalid_api_key",
                 "ORG_ID": "staging"}
        write_creds(creds)

        data = {'key': 'value'}
        notify(data, safe=True)

        captured = capsys.readouterr()

        assert captured.out.strip() == "[jovian] message_sent:False"


@mock.patch("requests.post", side_effect=mock_requests_post)
def test_notify_log_error(mock_requests_post, capsys):
    with fake_creds('.jovian-notify', 'credentials.json'):
        # setUp
        creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                 "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                 "API_URL": "https://api-staging.jovian.ai",
                 "API_KEY": "fake_expired_api_key",
                 "ORG_ID": "staging"}
        write_creds(creds)

        data = {'key': 'value'}
        notify(data)

        captured = capsys.readouterr()

        assert captured.err.strip() == "[jovian] Error: The token has expired"


class TestNotify(TestCase):
    @mock.patch("jovian.utils.request.get_api_key", return_value="fake_invalid_api_key")
    @mock.patch("requests.post", side_effect=mock_requests_post)
    def test_notify_safe_false_raises_api_error(self, mock_requests_post, mock_get_api_key):
        with fake_creds('.jovian-notify', 'credentials.json'):
            # setUp
            creds = {"WEBAPP_URL": "https://staging.jovian.ml/",
                     "GUEST_KEY": "b6538d4dfde04fcf993463a828a9cec6",
                     "API_URL": "https://api-staging.jovian.ai",
                     "API_KEY": "fake_invalid_api_key",
                     "ORG_ID": "staging"}
            write_creds(creds)

            data = {'key': 'value'}
            with self.assertRaises(ApiError):
                notify(data)



import os
from unittest import TestCase, mock

from jovian.utils.extension import setup_extension


class TestSetupExtension(TestCase):
    @mock.patch('os.system')
    def test_setup_extension_true(self, mock_system):
        setup_extension(True)
        mock_system.assert_called_with("jupyter nbextension enable jovian_nb_ext/main --sys-prefix")

    @mock.patch('os.system')
    def test_setup_extension_false(self, mock_system):
        setup_extension(False)
        mock_system.assert_called_with("jupyter nbextension disable jovian_nb_ext/main --sys-prefix")



import sys
from unittest import TestCase, mock

from jovian._version import __version__
from jovian.utils.constants import LINUX, MACOS, WINDOWS
from jovian.utils.misc import (get_file_extension, get_flavor, get_platform, is_flavor_pro, is_uuid, timestamp_ms,
                               urljoin, version)


class TestIsUUID(TestCase):

    def test_is_uuid_without_hyphens(self):
        text = "374ab6086ca449769b3e44b6e35f9126"

        self.assertTrue(is_uuid(text))

    def test_is_uuid_with_hypens(self):
        text = "374ab608-6ca4-4976-9b3e-44b6e35f9126"

        self.assertTrue(is_uuid(text))

    def test_is_uuid_false(self):
        text = "this-is-not-a-uuid"

        self.assertFalse(is_uuid(text))


class TestGetFileExtension(TestCase):

    def test_get_file_extension_normal(self):
        file = "users/siddhant/file.ipynb"
        expected_result = "ipynb"

        self.assertEqual(get_file_extension(file), expected_result)

    def test_get_file_extension_malformed(self):
        file = 123
        expected_result = ''

        self.assertEqual(get_file_extension(file), expected_result)


class TestGetPlatform(TestCase):
    @mock.patch("platform.system", mock.Mock(return_value="Windows"))
    def test_get_platform_windows(self):
        self.assertEqual(get_platform(), WINDOWS)

    @mock.patch("platform.system", mock.Mock(return_value="Linux"))
    def test_get_platform_linux(self):
        self.assertEqual(get_platform(), LINUX)

    @mock.patch("platform.system", mock.Mock(return_value="Darwin"))
    def test_get_platform_macos(self):
        self.assertEqual(get_platform(), MACOS)


class UrlUtilsTest(TestCase):

    def test_urljoin_normal(self):
        base_url_1 = "https://jovian.ml"
        base_url_2 = "https://jovian.ml/"

        path_1 = "user/siddhant"
        path_2 = "/user/siddhant"

        expected_result = "https://jovian.ml/user/siddhant"

        self.assertEqual(urljoin(base_url_1, path_1), expected_result)
        self.assertEqual(urljoin(base_url_1, path_2), expected_result)
        self.assertEqual(urljoin(base_url_2, path_1), expected_result)
        self.assertEqual(urljoin(base_url_2, path_2), expected_result)

    def test_urljoin_trailing_slash(self):
        base_url_1 = "https://jovian.ml"
        base_url_2 = "https://jovian.ml/"

        path_1 = "user/siddhant/"
        path_2 = "/user/siddhant/"

        expected_result = "https://jovian.ml/user/siddhant/"

        self.assertEqual(urljoin(base_url_1, path_1), expected_result)
        self.assertEqual(urljoin(base_url_1, path_2), expected_result)
        self.assertEqual(urljoin(base_url_2, path_1), expected_result)
        self.assertEqual(urljoin(base_url_2, path_2), expected_result)

    def test_urljoin_multiple(self):
        base_url_1 = "https://jovian.ml"

        path_1 = "user/siddhant/"
        path_2 = "/gists/starred"

        expected_result = "https://jovian.ml/user/siddhant/gists/starred"

        self.assertEqual(urljoin(base_url_1, path_1, path_2), expected_result)

    def test_urljoin_malformed(self):
        base_url_1 = "https://jovian.ml"
        base_url_2 = "https://jovian.ml/"

        path_1 = "///user/siddhant/"
        path_2 = "//user/siddhant//////"

        expected_result = "https://jovian.ml/user/siddhant/"

        self.assertEqual(urljoin(base_url_1, path_1), expected_result)
        self.assertEqual(urljoin(base_url_1, path_2), expected_result)
        self.assertEqual(urljoin(base_url_2, path_1), expected_result)
        self.assertEqual(urljoin(base_url_2, path_2), expected_result)

    def test_urljoin_no_args(self):
        with self.assertRaises(TypeError) as cm:
            urljoin()

        self.assertEqual(str(cm.exception), "urljoin requires at least one argument")

    def test_urljoin_single_url(self):
        base_url_1 = "https://jovian.ml"
        base_url_2 = "https://jovian.ml/"

        self.assertEqual(urljoin(base_url_1), base_url_1)
        self.assertEqual(urljoin(base_url_2), base_url_2)

    def test_urljoin_single_path(self):
        path_1 = "///user/siddhant"

        expected_result = "user/siddhant"

        self.assertEqual(urljoin(path_1), expected_result)
        self.assertEqual(urljoin(path_1), expected_result)


class TestTimestampMs(TestCase):
    @mock.patch('time.time', mock.Mock(return_value=1577112328.529031))
    def test_timestamp_ms(self):
        self.assertEqual(timestamp_ms(), 1577112328529)


class TestGetFlavor(TestCase):
    def test_get_flavor(self):
        self.assertEqual(get_flavor(), "jovian")

    def test_get_flavor_import_error(self):
        import builtins
        _original_import = builtins.__import__

        def error_raiser(*args, **kwargs):
            raise ImportError('fake import error')

        builtins.__import__ = error_raiser
        flavor = get_flavor()
        builtins.__import__ = _original_import

        self.assertEqual(flavor, "jovian")


class TestIsFlavorPro(TestCase):
    def test_is_flavor_pro_false(self):
        import jovian._flavor
        jovian._flavor.__flavor__ = 'jovian'
        self.assertEqual(is_flavor_pro(), False)

    def test_is_flavor_pro_true(self):
        import jovian._flavor
        jovian._flavor.__flavor__ = 'jovian-pro'
        self.assertEqual(is_flavor_pro(), True)


class TestVersion(TestCase):
    def test_version(self):
        expected_result = __version__
        self.assertEqual(version(), expected_result)



from unittest import TestCase, mock

import jovian.utils.records
from jovian.utils.records import get_records, log_dataset, log_git, log_hyperparams, log_metrics, log_record, reset


class FakeRecords(TestCase):
    def setUp(self):
        self._d = jovian.utils.records._data_blocks
        jovian.utils.records._data_blocks = [('fake_slug_metrics_1', 'metrics', {}),
                                             ('fake_slug_metrics_2', 'metrics', {}),
                                             ('fake_slug_hyperparams_1', 'hyperparams', {}),
                                             ('fake_slug_hyperparams_2', 'hyperparams', {})]

    def tearDown(self):
        jovian.utils.records._data_blocks = self._d


class TestGetRecords(FakeRecords):
    def test_get_records_without_type(self):
        expected_result = ['fake_slug_metrics_1',
                           'fake_slug_metrics_2',
                           'fake_slug_hyperparams_1',
                           'fake_slug_hyperparams_2']

        self.assertEqual(get_records(slug_only=True), expected_result)

    def test_get_records_with_type(self):
        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {}),
                           ('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {})]
        self.assertEqual(get_records(), expected_result)


class TestReset(FakeRecords):
    def test_reset(self):
        reset()
        expected_result = []
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)

    def test_reset_metrics(self):
        reset('metrics')
        expected_result = [('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {})]
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)

    def test_reset_hyperparams(self):
        reset('hyperparams')
        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {})]
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)


def mock_api_post_block(*args, **kwargs):
    data = {
        'count': 1,
        'tracking': {
            'createdAt': 1577270059593,
            'updatedAt': None, 'gistSlug': None,
            'trackingSlug': 'fake_slug_3',
            'gistVersion': None
        }
    }

    return data


class TestLogRecord(FakeRecords):
    @mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
    def test_log_record_no_data(self, mock_api_post_block):
        data = {}
        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {}),
                           ('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {})]

        log_record('fake_record_type', data)
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)


class TestLogHyperparams(FakeRecords):

    @mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
    def test_log_hyperparams(self, mock_api_post_block):
        data = {
            'arch_name': 'cnn_1',
            'lr': .001
        }
        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {}),
                           ('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {}),
                           ('fake_slug_3', 'hyperparams', data)]

        log_hyperparams(data)
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)


class TestLogMetrics(FakeRecords):

    @mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
    def test_log_metrics(self, mock_api_post_block):
        data = {'acc': 0.89, 'val_acc': 0.86}
        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {}),
                           ('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {}),
                           ('fake_slug_3', 'metrics', data)]

        log_metrics(acc=0.89, val_acc=0.86)
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)


class TestLogDataset(FakeRecords):

    @mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
    def test_log_dataset(self, mock_api_post_block):
        data = {
            'col1': [1, 2, 3],
            'col2': [4, 5, 6]
        }
        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {}),
                           ('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {}),
                           ('fake_slug_3', 'dataset', data)]

        log_dataset(data)
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)


class TestLogGit(FakeRecords):

    @mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
    def test_log_git(self, mock_api_post_block):
        data = {
            'branch': 'sample_branch',
            'commit': 'sample_commit_hash',
            'remote': 'https://github.com/rohitsanj/fakerepo'
        }

        expected_result = [('fake_slug_metrics_1', 'metrics', {}),
                           ('fake_slug_metrics_2', 'metrics', {}),
                           ('fake_slug_hyperparams_1', 'hyperparams', {}),
                           ('fake_slug_hyperparams_2', 'hyperparams', {}),
                           ('fake_slug_3', 'git', data)]

        log_git(data)
        self.assertEqual(jovian.utils.records._data_blocks, expected_result)


# Test verbose log of `log_hyperparams` and `log_metrics`

@mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
def test_log_hyperparams_verbose(mock_api_post_block, capsys):
    data = {
        'arch_name': 'cnn_1',
        'lr': .001
    }
    log_hyperparams(data)
    captured = capsys.readouterr()

    expected_result = '[jovian] Hyperparams logged.'
    assert expected_result == captured.out.strip()


@mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
def test_log_metrics_verbose(mock_api_post_block, capsys):
    data = {
        'acc': 0.89,
        'val_acc': 0.86
    }
    log_metrics(data)
    captured = capsys.readouterr()

    expected_result = '[jovian] Metrics logged.'
    assert expected_result == captured.out.strip()


@mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
def test_log_dataset_verbose(mock_api_post_block, capsys):
    data = {
        'col1': [1, 2, 3],
        'col2': [4, 5, 6]
    }
    log_dataset(data)
    captured = capsys.readouterr()

    expected_result = '[jovian] Dataset logged.'
    assert expected_result == captured.out.strip()


@mock.patch("jovian.utils.records.api.post_block", side_effect=mock_api_post_block)
def test_log_git_verbose(mock_api_post_block, capsys):
    data = {
        'branch': 'sample_branch',
        'commit': 'sample_commit_hash',
        'remote': 'https://github.com/rohitsanj/fakerepo'
    }
    log_git(data)
    captured = capsys.readouterr()

    expected_result = '[jovian] Git logged.'
    assert expected_result == captured.out.strip()



from jovian.utils.error import ApiError, CondaError, ConfigError


def test_conda_error():
    msg = 'This is a error'
    e = CondaError(msg)
    assert e.args == (msg,)


def test_api_error():
    msg = 'This is a error'
    e = ApiError(msg)
    assert e.args == (msg,)


def test_config_error():
    msg = 'This is a error'
    e = ConfigError(msg)
    assert e.args == (msg,)



from unittest import TestCase, mock
from jovian.utils.script import get_script_filename, in_script


class TestGetScriptFilename(TestCase):

    def test_get_script_filename_normal(self):
        import __main__
        __main__.__file__ = 'sample.py'

        expected_result = 'sample.py'
        self.assertEqual(get_script_filename(), expected_result)

    @mock.patch('builtins.__import__', return_value=ImportError('fake import error'))
    def test_get_script_filename_exception(self, mock_import):

        expected_result = None
        self.assertEqual(get_script_filename(), expected_result)


class TestInScript(TestCase):

    def test_in_script_valid_extension(self):
        import __main__
        __main__.__file__ = 'sample.py'

        self.assertTrue(in_script())

    def test_in_script_invalid_extension(self):
        import __main__
        __main__.__file__ = 'sample.txt'

        self.assertFalse(in_script())

    def test_in_script_false(self):
        import __main__
        __main__.__file__ = None

        self.assertFalse(in_script())



import json
import os
import shutil
from unittest import TestCase, mock

from jovian.utils.constants import RC_FILENAME
from jovian.utils.rcfile import (get_notebook_slug, get_rcdata, make_rcdata, rcfile_exists, save_rcdata,
                                 set_notebook_slug)

data = {
    "notebooks": {
        "Testing Jovian.ipynb": {
            "slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"
        }
    }
}


class RCFile(TestCase):
    path = 'rcfile-testing-folder'

    def setUp(self):
        os.mkdir(self.path)
        os.chdir(self.path)

    def tearDown(self):
        os.chdir('..')
        shutil.rmtree(self.path)


class CreateNewRCFile(RCFile):
    def setUp(self):
        super().setUp()
        with open(RC_FILENAME, 'w') as f:
            json.dump(data, f, indent=2)

    def tearDown(self):
        os.system('rm .jovianrc')
        super().tearDown()


class TestRCFileDoesNotExist(RCFile):

    def test_rcfile_does_not_exist(self):
        self.assertFalse(rcfile_exists())


class TestSaveRCData(RCFile):

    def test_save_rcdata_data_none(self):
        save_rcdata(data=None)
        expected_result = {
            "notebooks": {}
        }
        with open(RC_FILENAME, 'r') as f:
            self.assertEqual(json.load(f), expected_result)

    def test_save_rcdata_data_exists(self):
        save_rcdata(data=data)
        expected_result = data

        with open(RC_FILENAME, 'r') as f:
            self.assertEqual(json.load(f), expected_result)


class TestGetRCDataRCFileNotExist(RCFile):

    @mock.patch('jovian.utils.rcfile.rcfile_exists', mock.Mock(return_value=False))
    def test_get_rcdata_rcfile_does_not_exist(self):
        expected_result = {
            "notebooks": {}
        }
        self.assertEqual(get_rcdata(), expected_result)


class TestGetRCData(RCFile):

    def test_get_rcdata_rcfile_exists(self):
        with open(RC_FILENAME, 'w') as f:
            json.dump(data, f, indent=2)

        expected_result = data

        self.assertEqual(get_rcdata(), expected_result)

    def tearDown(self):
        os.system('rm .jovianrc')
        super().tearDown()


class TestGetNotebookSlug(CreateNewRCFile):

    def test_get_notebook_slug_notebook_present(self):
        filename = "Testing Jovian.ipynb"
        expected_result = "46bd9a3f87e74de0baf8a6f0b60a8df9"

        self.assertEqual(get_notebook_slug(filename), expected_result)

    def test_get_notebook_slug_notebook_not_present(self):
        filename = "Testing Jovian123.ipynb"
        expected_result = None

        self.assertEqual(get_notebook_slug(filename), expected_result)


class TestSetNotebookSlug(CreateNewRCFile):

    def test_set_notebook_slug_new_entry(self):
        filename = "Testing Jovian 2.ipynb"
        slug = "46bd9a3f87e74de0baf8a6f0b60a8df9"

        expected_result = {
            "notebooks": {
                "Testing Jovian.ipynb": {
                    "slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"
                },
                "Testing Jovian 2.ipynb": {
                    "slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"
                }
            }
        }

        set_notebook_slug(filename, slug)
        self.assertEqual(get_rcdata(), expected_result)

    def test_set_notebook_slug_existing_entry(self):
        filename = "Testing Jovian.ipynb"
        slug = "46bd9a3f87e74de0baf8a6f0b60a8df9"

        expected_result = data

        set_notebook_slug(filename, slug)
        self.assertEqual(get_rcdata(), expected_result)


class TestMakeRCData(CreateNewRCFile):
    def test_make_rcdata(self):
        filename = "Testing Jovian.ipynb"
        slug = "46bd9a3f87e74de0baf8a6f0b60a8df9"
        expected_result = '{"notebooks": {"Testing Jovian.ipynb": {"slug": "46bd9a3f87e74de0baf8a6f0b60a8df9"}}}'

        self.assertEqual(make_rcdata(filename, slug), expected_result)



import os
from unittest import TestCase, mock

import yaml

from jovian.utils.envfile import (check_error, check_pip_failed, dump_environment_to_yaml_file, extract_env_name,
                                  extract_env_packages, extract_package_from_line, extract_pip_packages,
                                  get_environment_dict, identify_env_file, remove_packages, request_env_name,
                                  sanitize_envfile, serialize_packages, write_env_name)


class EnvFile(TestCase):
    def setUp(self):
        os.chdir('jovian/tests/resources/yaml')

    def tearDown(self):
        os.chdir('../' * 4)


class TestIdentityEnvFile(EnvFile):

    def test_identify_env_file_no_environment_file(self):
        self.assertIsNone(identify_env_file(env_fname=None))

    def test_identify_env_file_exists(self):
        os.system('touch environment.yml')

        self.assertEqual(identify_env_file(env_fname=None), 'environment.yml')

        os.system('rm environment.yml')


class TestGetEnvironmentDict(EnvFile):
    def test_get_environment_dict(self):
        expected_result = {'prefix': '/home/admin/anaconda3/envs/test-env',
                           'dependencies': ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                            {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
                           'channels': ['defaults'], 'name': 'test-env'}
        environment = get_environment_dict(env_fname='environment-test.yml')

        self.assertDictEqual(environment, expected_result)

    def test_get_environment_dict_non_existent_file(self):
        expected = {'prefix': '/home/admin/anaconda3/envs/test-env',
                    'dependencies': ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                     {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
                    'channels': ['defaults'], 'name': 'test-env'}
        environment = get_environment_dict(env_fname='environment-test.yml')

        with self.assertRaises(FileNotFoundError):
            get_environment_dict(env_fname='non-existent-file.yml')

    def test_get_environment_dict_raises_exception_invalid_file(self):
        expected_result = {}

        self.assertDictEqual(get_environment_dict('invalid-yaml-file.yml'), expected_result)


class TestExtractEnvName(EnvFile):
    def test_extract_env_name(self):
        name = extract_env_name(env_fname='environment-test.yml')
        name2 = extract_env_name(env_fname='empty-yaml-file.yml')

        self.assertEqual(name, 'test-env')
        self.assertIsNone(name2)
        with self.assertRaises(FileNotFoundError):
            extract_env_name(env_fname='non-existent-file.yml')


class TestExtractEnvPackages(EnvFile):
    def test_extract_env_packages(self):

        dep = extract_env_packages(env_fname='environment-test.yml')
        dep2 = extract_env_packages(env_fname='empty-yaml-file.yml')

        self.assertListEqual(dep, ['mixpanel=1.11.0', 'sigmasix=1.91.0', 'sqlite',
                                   'six==1.11.0', 'sqlite==2.0.0'])
        self.assertListEqual(dep2, [])
        with self.assertRaises(FileNotFoundError):
            extract_env_packages(env_fname='non-existent-file.yml')


class TestExtractPipPackages(EnvFile):
    def test_extract_pip_packages(self):
        dep = extract_pip_packages(env_fname='environment-test.yml')
        dep2 = extract_pip_packages(env_fname='empty-yaml-file.yml')

        self.assertListEqual(dep, ['six==1.11.0', 'sqlite==2.0.0'])
        self.assertListEqual(dep2, [])
        with self.assertRaises(FileNotFoundError):
            extract_env_packages(env_fname='non-existent-file.yml')


class TestCheckError(EnvFile):
    def test_check_error(self):

        packages = extract_env_packages(env_fname='environment-test.yml')

        error_str = '''ResolvePackageNotFound: 
                        - mixpanel=1.11.0'''
        error_str2 = '''UnsatisfiableError: 
                                - sigmasix=1.91.0'''
        error_str3 = '''AnyOtherException: 
                                - sigmasix=1.91.0'''
        error, pkgs = check_error(error_str=error_str, packages=packages)
        error2, pkgs2 = check_error(error_str=error_str2, packages=packages)
        error3, pkgs3 = check_error(error_str=error_str3, packages=packages)

        self.assertEqual(error, 'unresolved')
        self.assertListEqual(pkgs, ['mixpanel=1.11.0'])
        self.assertEqual(error2, 'unsatisfiable')
        self.assertListEqual(pkgs2, ['sigmasix=1.91.0'])
        self.assertIsNone(error3)
        self.assertListEqual(pkgs3, [])

    def test_check_error_no_packages(self):
        error_str = '''ResolvePackageNotFound: 
                        - mixpanel=1.11.0'''
        packages = []
        error, pkgs = check_error(error_str=error_str, packages=packages)

        self.assertEqual(error, 'unresolved')
        self.assertListEqual(pkgs, [])


class TestExtractPackageFromLine(EnvFile):
    def test_extract_package_from_line(self):
        packages = extract_env_packages(env_fname='environment-test.yml')

        lines = [['- mixpanel=1.11.0', 'mixpanel=1.11.0'], ['sqlite<3x not compatible', 'sqlite'],
                 ['', None], ['line containing six < 3.0.x', 'six==1.11.0']]
        for line in lines:
            self.assertEqual(extract_package_from_line(line=line[0], packages=packages), line[1])


class TestDumpEnvironmentToYamlFile(EnvFile):
    data = {'channels': ['defaults'],
            'dependencies': ['mixpanel=1.11.0',
                             'sigmasix=1.91.0',
                             'sqlite',
                             {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
            'name': 'test-env',
            'prefix': '/home/admin/anaconda3/envs/test-env'}

    def setUp(self):
        super().setUp()
        dump_environment_to_yaml_file('test.yml', self.data)

    def test_dump_environment_to_yaml_file_normal(self):
        self.assertEqual(get_environment_dict('test.yml'), self.data)

    def tearDown(self):
        os.system('rm test.yml')
        super().tearDown()


class TestWriteEnvName(EnvFile):

    def test_write_env_name_new_name(self):
        data = {'channels': ['defaults'],
                'dependencies': ['mixpanel=1.11.0'],
                'name': 'test-env',
                'prefix': '/home/admin/anaconda3/envs/test-env'}
        dump_environment_to_yaml_file('test.yml', data)

        write_env_name('test-env-changed', 'test.yml')

        expected_result = 'test-env-changed'
        self.assertEqual(get_environment_dict('test.yml')['name'], expected_result)

    def test_write_env_name_no_name(self):
        data = {'channels': ['defaults'],
                'dependencies': ['mixpanel=1.11.0'],
                'prefix': '/home/admin/anaconda3/envs/test-env'}
        dump_environment_to_yaml_file('test.yml', data)

        write_env_name('test-env', 'test.yml')

        expected_result = 'test-env'
        self.assertEqual(get_environment_dict('test.yml')['name'], expected_result)

    def test_write_env_name_same_name(self):
        data = {'channels': ['defaults'],
                'dependencies': ['mixpanel=1.11.0'],
                'name': 'test-env',
                'prefix': '/home/admin/anaconda3/envs/test-env'}
        dump_environment_to_yaml_file('test.yml', data)

        write_env_name('test-env', 'test.yml')

        expected_result = 'test-env'
        self.assertEqual(get_environment_dict('test.yml')['name'], expected_result)

    def tearDown(self):
        os.system('rm test.yml')
        super().tearDown()


class TestRemovePackages(EnvFile):

    def test_remove_packages(self):
        packages = extract_env_packages(env_fname='environment-test.yml')
        dependencies = ['dep1', 'dep2', 'mixpanel=1.11.0', 'sqlite']

        expected_result = ['dep1', 'dep2']

        self.assertEqual(remove_packages(dependencies, packages), expected_result)

    def test_remove_packages_pip(self):
        packages = extract_env_packages(env_fname='environment-test.yml')
        dependencies = ['dep1', 'mixpanel=1.11.0', {'pip': ['six==1.11.0', 'dep3']}]

        expected_result = ['dep1', {'pip': ['dep3']}]

        self.assertEqual(remove_packages(dependencies, packages), expected_result)


class TestSanitizeEnvfile(EnvFile):
    data = {'channels': ['defaults'],
            'dependencies': ['mixpanel=1.11.0',
                             'sigmasix=1.91.0',
                             'sqlite',
                             {'pip': ['six==1.11.0', 'sqlite==2.0.0']}],
            'name': 'test-env',
            'prefix': '/home/admin/anaconda3/envs/test-env'}

    def setUp(self):
        super().setUp()
        dump_environment_to_yaml_file('test.yml', self.data)

    def test_sanitize_envfile(self):
        packages = ['six==1.11.0', 'sigmasix=1.91.0']
        sanitize_envfile('test.yml', packages)

        expected_result = {'channels': ['defaults'],
                           'dependencies': ['mixpanel=1.11.0',
                                            'sqlite',
                                            {'pip': ['sqlite==2.0.0']}],
                           'name': 'test-env',
                           'prefix': '/home/admin/anaconda3/envs/test-env'}
        self.assertEqual(get_environment_dict('test.yml'), expected_result)

    def tearDown(self):
        os.system('rm test.yml')
        super().tearDown()


class TestSerializePackages(EnvFile):

    def test_serialize_packages(self):
        dependencies = ['dep1', 'mixpanel=1.11.0', {'pip': ['six==1.11.0', 'dep3']}]

        expected_result = ['dep1', 'mixpanel=1.11.0', 'six==1.11.0', 'dep3']

        self.assertEqual(serialize_packages(dependencies), expected_result)


class TestRequestEnvName(EnvFile):
    def tearDown(self):
        write_env_name('test-env', 'environment-test.yml')
        super().tearDown()

    @mock.patch("jovian.utils.envfile.click.prompt", return_value="")
    def test_request_env_name(self, mock_prompt):
        expected_result = 'test-env'

        self.assertEqual(request_env_name(env_name=None, env_fname='environment-test.yml'), expected_result)

    @mock.patch("jovian.utils.envfile.click.prompt", return_value="test-env-changed")
    def test_request_env_name_changed(self, mock_prompt):
        expected_result = 'test-env-changed'

        self.assertEqual(request_env_name(env_name=None, env_fname='environment-test.yml'), expected_result)

    @mock.patch("jovian.utils.envfile.extract_env_name", return_value=None)
    @mock.patch("jovian.utils.envfile.click.prompt", return_value="test-env-changed")
    def test_request_env_name_no_env_name(self, mock_prompt, mock_extract_env_name):
        expected_result = 'test-env-changed'

        self.assertEqual(request_env_name(env_name=None, env_fname='environment-test.yml'), expected_result)

    @mock.patch("jovian.utils.envfile.extract_env_name", return_value=None)
    @mock.patch("jovian.utils.envfile.click.prompt", return_value="")
    def test_request_env_name_no_env_name_default_to_base(self, mock_prompt, mock_extract_env_name):
        expected_result = 'base'

        self.assertEqual(request_env_name(env_name=None, env_fname='environment-test.yml'), expected_result)


class TestCheckPipFailed(TestCase):
    def test_check_pip_failed_true(self):
        error_str = '''Could not install
                       Pip failed with error code 2'''

        self.assertTrue(check_pip_failed(error_str))

    def test_check_pip_failed_false(self):
        error_str = '''Pip successfully installed package'''

        self.assertFalse(check_pip_failed(error_str))
